/**********************************************************************
 *
 * Copyright (c) 2004 Olaf Willuhn
 * All rights reserved.
 * 
 * This software is copyrighted work licensed under the terms of the
 * Jameica License.  Please consult the file "LICENSE" for details. 
 *
 **********************************************************************/

package de.willuhn.jameica.hbci.synchronize.scripting;

import de.willuhn.jameica.hbci.synchronize.SynchronizeJobProvider;


/**
 * Marker-Interface, um die eigenen Job-Provider zu finden.
 */
public interface ScriptingSynchronizeJobProvider extends SynchronizeJobProvider
{
}


