/**********************************************************************
 *
 * Copyright (c) 2004 Olaf Willuhn
 * All rights reserved.
 * 
 * This software is copyrighted work licensed under the terms of the
 * Jameica License.  Please consult the file "LICENSE" for details. 
 *
 **********************************************************************/
package de.willuhn.jameica.hbci.rmi;


/**
 * Ne Lastschrift ist ja auch nur ne Ueberweisung. Nur andersrum ;).
 */
public interface Lastschrift extends BaseUeberweisung
{
}
